# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from pyams_content_es.interfaces import IContentIndexerUtility
from pyams_elastic.include import client_from_config
from pyams_utils.factory import create_object
from pyams_utils.interfaces.monitor import IMonitorExtension, IMonitorExtensionStatus
from pyams_utils.registry import query_utility, utility_config

__docformat__ = 'restructuredtext'


@utility_config(name='pyams_content_es.monitor',
                provides=IMonitorExtension)
class ElasticsearchContentMonitor:
    """Elasticsearch indexer monitor utility"""

    def get_status(self, request):
        registry = request.registry
        es_client = client_from_config(registry.settings)
        if not es_client.es.ping():
            yield create_object(IMonitorExtensionStatus,
                                handler='pyams_content_es.monitor:client',
                                status='DOWN',
                                message='Elasticsearch server not available!')
        else:
            yield create_object(IMonitorExtensionStatus,
                                handler='pyams_content_es.monitor:client',
                                status='UP')
            indexer = query_utility(IContentIndexerUtility)
            if indexer is None:
                yield create_object(IMonitorExtensionStatus,
                                    handler='pyams_content_es.monitor:indexer',
                                    status='DOWN',
                                    message='Missing content indexer utility!')
            else:
                yield create_object(IMonitorExtensionStatus,
                                    handler='pyams_content_es.monitor:indexer',
                                    status='UP')
                status_code, message = indexer.test_process()
                if status_code == 200:
                    yield create_object(IMonitorExtensionStatus,
                                        handler='pyams_content_es.monitor:indexer:process',
                                        status='UP',
                                        message=message)
                else:
                    yield create_object(IMonitorExtensionStatus,
                                        handler='pyams_content_es.monitor:indexer:process',
                                        status='DOWN',
                                        message=f'Content indexer process not running: {message}')



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content_es.shared.view.interfaces module

This module defines interfaces related to views management.
"""

from pyams_content.shared.view import IViewQuery
from pyams_content.shared.view.interfaces.query import IViewQueryResultsFilterExtension, IViewQueryParamsExtension, \
    IViewUserQuery

__docformat__ = 'restructuredtext'


class IEsViewQuery(IViewQuery):
    """Elasticsearch view query marker interface

    This is the base interface of view query.
    """

    def get_post_filters(self, context, request=None, **kwargs):
        """Get static view query post-filters params

        These filters are defined as query parameters, but are applied only after
        aggregates calculation.
        """


class IEsViewQueryParamsExtension(IViewQueryParamsExtension):
    """Elasticsearch view query params extension

    This interface is used to register custom adapters which are defined to get
    Elasticsearch query extra parameters.
    """


class IEsViewUserQuery(IViewUserQuery):
    """Elasticsearch view user query interface"""


class IEsViewQueryPostFilterExtension(IViewQueryParamsExtension):
    """Elasticsearch view query post-filter extension

    This interface is used to register custom adapters which are defined to get
    Elasticsearch post-filter extra parameters; these filters are used to
    filter results after aggregates calculation.
    """


class IEsViewUserPostFilter(IViewUserQuery):
    """Elasticsearch view user post-filter interface"""


class IEsViewQueryResultsFilterExtension(IViewQueryResultsFilterExtension):
    """Elasticsearch view query filter extension

    This interface is used to register adapters which are defined to
    filter results of Elasticsearch query. So unlike params extensions,
    they are generally slower and can alter the list of query results.
    """



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content_es.component.extfile module

This module defines adapters which are used to handle external file indexation.
"""

import base64

from pyams_content.component.extfile import IExtFile
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content_es.interfaces import IDocumentIndexInfo
from pyams_utils.adapter import adapter_config
from pyams_utils.finder import find_objects_providing
from pyams_utils.traversing import get_parent
from pyams_workflow.interfaces import IWorkflow, IWorkflowState

__docformat__ = 'restructuredtext'


@adapter_config(name='extfile',
                required=IParagraphContainerTarget,
                provides=IDocumentIndexInfo)
def paragraph_container_extfile_index_info(context):
    """Paragraph container external file indexation info"""
    extfiles = []
    attachments = []
    workflow_state = None
    workflow = IWorkflow(context, None)
    if workflow is not None:
        workflow_state = IWorkflowState(context, None)
    # don't index attachments for contents which are not published
    if (workflow_state is None) or (workflow_state.state in workflow.visible_states):
        max_file_size = getattr(context, '_v_es_max_file_size', 0) * 1024
        # extract attachments
        for extfile in find_objects_providing(context, IExtFile):
            if not extfile.visible:
                continue
            paragraph = get_parent(extfile, IBaseParagraph)
            if (paragraph is not None) and not paragraph.visible:
                continue
            extfiles.append({
                'title': extfile.title,
                'description': extfile.description
            })
            if not extfile.data:
                continue
            for lang, data in extfile.data.items():
                if max_file_size and (data.get_size() > max_file_size):
                    continue
                content_type = data.content_type
                if isinstance(content_type, bytes):
                    content_type = content_type.decode()
                if content_type.startswith('image/') or \
                        content_type.startswith('audio/') or \
                        content_type.startswith('video/'):
                    continue
                attachments.append({
                    'content_type': content_type,
                    'name': data.filename,
                    'language': lang,
                    'content': base64.encodebytes(data.data).decode().replace('\n', '')
                })
    result = {
        'extfile': extfiles
    }
    if attachments:
        result.update({
            '__pipeline__': 'attachment',
            'attachments': attachments
        })
    return result



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from elasticsearch_dsl import A, AttrDict, Search

from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER
from pyams_content.shared.view.interfaces.query import IViewQuery
from pyams_elastic.include import get_client
from pyams_utils.list import unique_iter
from pyams_utils.request import query_request

__docformat__ = 'restructuredtext'


def get_merged_results(views, context, request=None, get_user_params=False, aggregates=None,
                       sort_index=None, reverse=False, limit=None, **kwargs):
    """Get results from merged views"""
    if request is None:
        request = query_request()
    client = get_client(request)
    search_params = []
    for view in views:
        query = IViewQuery(view, None)
        if query is not None:
            params = query.get_params(context, request, get_user_params=get_user_params, **kwargs)
            if params is None:
                continue
            search_params.append(Search().query(params).to_dict()['query'])
    if not search_params:
        return (), 0, {}
    combined_params = {'bool': {'should': search_params}}
    search = Search(using=client.es, index=client.index) \
        .params(request_timeout=30) \
        .query(combined_params) \
        .source(['internal_id'])
    if aggregates:
        for agg in aggregates:
            search.aggs.bucket(agg['name'],
                               A(agg['type'], **agg['params']))
    # Define sort order
    sort_values = []
    if (not sort_index) or (sort_index == RELEVANCE_ORDER):
        sort_values.append({
            '_score': {
                'order': 'desc'
            }
        })
    else:
        sort_values.append({
            'workflow.{0}'.format(sort_index): {
                'order': 'desc' if reverse else 'asc',
                'unmapped_type': 'date'
            }
        })
    if sort_values:
        search = search.sort(*sort_values)
    if limit:
        search = search[:limit]
    else:
        search = search[:999]
    results = search.execute()
    items = CatalogResultSet([result.internal_id for result in results.hits])
    aggregations = results.aggregations
    total_count = results.hits.total
    if isinstance(total_count, (dict, AttrDict)):
        total_count = results.hits.total['value']
    return unique_iter(items), total_count, aggregations
